#include <math.h>
#include <util/delay.h>
#include <compat/twi.h>
#include "i2cmaster.h"

#include "nunchuk.h"


uint8_t nunchuk_init ()
{
	uint8_t res;

  	// Uses New style init - no longer encrypted so no need to XOR bytes later... might save some cycles

	res = i2c_start((NUNCHUK_ADDR<<1)|TW_WRITE);
	if (! res)
	{
		return res;
	};

	i2c_write(0xF0);
	i2c_write(0x55);
	i2c_stop();

	_delay_ms(1);

	res = i2c_start((NUNCHUK_ADDR<<1)|TW_WRITE);
	if (! res)
	{
		return res;
	};

	i2c_write(0xFB);
	i2c_write(0x00);
	i2c_stop();

	_delay_ms(1);

	return 0;

}

uint8_t nunchuk_read(nunchuk_data_t *nunchuk_data)
{
	uint8_t res;
	uint8_t nc_data[6];

	nc_data[0] = 0;
	//res = twi_writeTo(NUNCHUK_ADDR, nc_data, 1, 1);
	//if (res != 0)
	//	return 1;

	i2c_start_wait((NUNCHUK_ADDR<<1)|TW_WRITE);
	i2c_write(0x00);
	i2c_stop();

	_delay_us(500); // the nunchuk needs some time to process
	
	//res = twi_readFrom(NUNCHUK_ADDR, nc_data, 7);
	//if (res != 7)
	//	return 2;

	i2c_start_wait((NUNCHUK_ADDR<<1)|TW_READ); 
	nc_data[0] = i2c_readAck();
	nc_data[1] = i2c_readAck();
	nc_data[2] = i2c_readAck();
	nc_data[3] = i2c_readAck();
	nc_data[4] = i2c_readAck();
	nc_data[5] = i2c_readNak();
	i2c_stop();

	nunchuk_data->x = nc_data[0];
	nunchuk_data->y = nc_data[1];
	nunchuk_data->acc_x = nc_data[2]<<2;
	nunchuk_data->acc_y = nc_data[3]<<2;
	nunchuk_data->acc_z = nc_data[4]<<2;
	nunchuk_data->but_z = ~nc_data[5] & 0x01;
	nunchuk_data->but_c = (~nc_data[5] >> 1) & 0x01;

	nunchuk_data->acc_x |= (nc_data[5] >> 2 ) & 0x3;
	nunchuk_data->acc_y |= (nc_data[5] >> 4 ) & 0x3;
	nunchuk_data->acc_z |= (nc_data[5] >> 6 ) & 0x3;

    return 0;
}

uint8_t roll_vals_idx = 0;
int16_t roll_vals[8];

uint8_t pitch_vals_idx = 0;
int16_t pitch_vals[8];

void nunchuk_calc_axis(const nunchuk_data_t *nunchuk_data, nunchuk_axis_t *nunchuk_axis)
{
	uint8_t i;

    int16_t x = nunchuk_data->acc_x - ZERO_ACC_X;
    int16_t y = nunchuk_data->acc_y - ZERO_ACC_Y;
    int16_t z = nunchuk_data->acc_z - ZERO_ACC_Z;

	if (z < 0)
		z = -z;


	float roll_raw = atan((float)x / (float)z);

	uint16_t roll = roll_raw * (180.0 / M_PI);
	//uint16_t pitch = atan((float)y / (float)z *cos(roll_raw)) * (180.0 / M_PI);
	uint16_t pitch = atan((float)y / (float)z) * (180.0 / M_PI);



//    - 
/*
	float yzmag = sqrt(y*y + z*z);
	uint16_t roll = atan2(x,yzmag) * (180.0 / M_PI); // "roll" angle

	float xzmag = sqrt(x*x + z*z);
	uint16_t pitch = atan2(y,xzmag) * (180.0 / M_PI); // "pitch" angle
*/

    roll_vals[roll_vals_idx] = roll;
    roll_vals_idx ++;
    if (roll_vals_idx > 7)
        roll_vals_idx = 0;

    int16_t roll_avg = 0;
	for (i=0; i<8; i++)
		roll_avg += roll_vals[i];
	roll_avg = roll_avg >> 3;
	//roll_avg = (roll_vals[0] + roll_vals[1] + roll_vals[2] + roll_vals[3]) >> 2;

    pitch_vals[pitch_vals_idx] = pitch;
    pitch_vals_idx ++;
    if (pitch_vals_idx > 7)
        pitch_vals_idx = 0;

	int16_t pitch_avg = 0;
	for (i=0; i<8; i++)
		pitch_avg += pitch_vals[i];
	pitch_avg = pitch_avg >> 3;
	//int16_t pitch_avg = (pitch_vals[0] + pitch_vals[1] + pitch_vals[2] + pitch_vals[3]) >> 2;


    nunchuk_axis->x = nunchuk_data->x - ZERO_X;
    nunchuk_axis->y = nunchuk_data->y - ZERO_Y;
    nunchuk_axis->roll = roll_avg;
    nunchuk_axis->pitch = pitch_avg;
    nunchuk_axis->but_c = nunchuk_data->but_c;
    nunchuk_axis->but_z = nunchuk_data->but_z;
}